//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Analog.asm
//**       Project:  Mini Sumo Project
//**       Created:  18 Jan 2009 by Josh Sparrow
//**       Revised:  12 Feb 2009 by Josh Sparrow
//**
//**   Description:  Routine to get information from the ADC.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Update_Analog
//	This routine is used to store the information
//	from the ADC to 4 words in ram. This also takes
//	the average of 5 samples.

Update_Analog:

	//Store registers
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3
	st	r6, sp, 4
	add	sp, sp, 5


	//Clear the accumulators
	mov	r0,0
	st	r0,IRdata0a
	st	r0,IRdata1a
	st	r0,CSdata0a
	st	r0,CSdata1a

	//Get the data from the ADC
AN_Pole0:
	inp	r0,ADCdata		//Wait for data to be ready
	and	r1,r0,0x8000
	bc	ZC,AN_Pole0
	rol	r1,r0,-13		//Determine if the data is for AN0, AN1, AN2 or AN3
	and	r1,r1,3
	bc	ZS,AN_Read0
	bra	AN_Pole0

AN_Start:	
	inp	r0,ADCdata		//Wait for data to be ready
	and	r1,r0,0x8000
	bc	ZC,AN_Start

	rol	r1,r0,-13		//Determine if the data is for AN0, AN1, AN2 or AN3
	and	r1,r1,3
	bc	ZS,AN_Read0
	
	add	r1,r1,-1
	bc	ZS,AN_Read1
	
	add	r1,r1,-1
	bc	ZS,AN_Read2
	
	add	r1,r1,-1
	bc	ZS,AN_Read3
	bra	AN_Start

	//This part will take and average of 5 samples of AN0
AN_Read0:
	and	r0,r0,0x03FF
	ld	r1,IRdata0a		//Get the accumulator data
	and	r2,r1,0x1FFF
	rol	r3,r1,-13		//Determine the number of samples that have been taken
	and	r3,r3,0x7

	add	r3,r3,-1
	bc	NC,AN_Read0a	//Branch if the first sample has been taken
	
	mov	r3,1			//Set the sample number to 1 and store the first sample
	bra	AN_Read0_S

AN_Read0a:
	add	r3,r3,-1
	bc	NC,AN_Read0b	//Branch if the second sample has been taken
                                                                               
	mov	r3,2			//Set the sample number to 2 and add the second sample
	bra	AN_Read0_S

AN_Read0b:
	add	r3,r3,-1
	bc	NC,AN_Read0c	//Branch if the third sample has been taken

	mov	r3,3			//Set the sample number to 3 and add the third sample
	bra	AN_Read0_S

AN_Read0c:
	add	r3,r3,-1
	bc	NC,AN_Read0d	//Branch if the forth sample has been taken

	mov	r3,4			//Set the sample number to 4 and add the forth sample
	bra	AN_Read0_S

AN_Read0d:
	mov	r3,0			//Clear the accumulator
	st	r3,IRdata0a
	
	add	r1,r2,r0		//Add final sample

	mov	r2,5			//Take the average
	jsr	r6,UnsignedDivide	
	and	r1,r1,0x3FF
	st	r1,IRdata0b
	bra	AN_Start

AN_Read0_S:	
	rol	r3,r3,13		//Prepare the data for the accumulator
	add	r2,r2,r0
	add	r3,r3,r2
	st	r3,IRdata0a
	bra	AN_Start

	//This part will take and average of 5 samples of AN0
AN_Read1:
	and	r0,r0,0x03FF
	ld	r1,IRdata1a		//Get the accumulator data
	and	r2,r1,0x1FFF
	rol	r3,r1,-13		//Determine the number of samples that have been taken
	and	r3,r3,0x7

	add	r3,r3,-1
	bc	NC,AN_Read1a	//Branch if the first sample has been taken
	
	mov	r3,1			//Set the sample number to 1 and store the first sample
	bra	AN_Read1_S

AN_Read1a:
	add	r3,r3,-1
	bc	NC,AN_Read1b	//Branch if the second sample has been taken
                                                                               
	mov	r3,2			//Set the sample number to 2 and add the second sample
	bra	AN_Read1_S

AN_Read1b:
	add	r3,r3,-1
	bc	NC,AN_Read1c	//Branch if the third sample has been taken

	mov	r3,3			//Set the sample number to 3 and add the third sample
	bra	AN_Read1_S

AN_Read1c:
	add	r3,r3,-1
	bc	NC,AN_Read1d	//Branch if the forth sample has been taken

	mov	r3,5			//Set the sample number to 4 and add the forth sample
	bra	AN_Read1_S

AN_Read1d:
	mov	r3,0			//Clear the accumulator
	st	r3,IRdata1a
	
	add	r1,r2,r0		//Add final sample

	mov	r2,5			//Take the average
	jsr	r6,UnsignedDivide	
	and	r1,r1,0x3FF
	st	r1,IRdata1b
	bra	AN_Start

AN_Read1_S:	
	rol	r3,r3,13		//Prepare the data for the accumulator
	add	r2,r2,r0
	add	r3,r3,r2
	st	r3,IRdata1a
	bra	AN_Start

	//This part will take and average of 5 samples of AN0
AN_Read2:
	and	r0,r0,0x03FF
	ld	r1,CSdata0a		//Get the accumulator data
	and	r2,r1,0x1FFF
	rol	r3,r1,-13		//Determine the number of samples that have been taken
	and	r3,r3,0x7

	add	r3,r3,-1
	bc	NC,AN_Read2a	//Branch if the first sample has been taken
	
	mov	r3,1			//Set the sample number to 1 and store the first sample
	bra	AN_Read2_S

AN_Read2a:
	add	r3,r3,-1
	bc	NC,AN_Read2b	//Branch if the second sample has been taken
                                                                               
	mov	r3,2			//Set the sample number to 2 and add the second sample
	bra	AN_Read2_S

AN_Read2b:
	add	r3,r3,-1
	bc	NC,AN_Read2c	//Branch if the third sample has been taken

	mov	r3,3			//Set the sample number to 3 and add the third sample
	bra	AN_Read2_S

AN_Read2c:
	add	r3,r3,-1
	bc	NC,AN_Read2d	//Branch if the forth sample has been taken

	mov	r3,4			//Set the sample number to 4 and add the forth sample
	bra	AN_Read2_S

AN_Read2d:
	mov	r3,0			//Clear the accumulator
	st	r3,CSdata0a
	
	add	r1,r2,r0		//Add final sample

	mov	r2,5			//Take the average
	jsr	r6,UnsignedDivide	
	and	r1,r1,0x3FF
	st	r1,CSdata0
	bra	AN_Start

AN_Read2_S:	
	rol	r3,r3,13		//Prepare the data for the accumulator
	add	r2,r2,r0
	add	r3,r3,r2
	st	r3,CSdata0a
	bra	AN_Start

	//This part will take and average of 5 samples of AN0
AN_Read3:
	and	r0,r0,0x03FF
	ld	r1,CSdata1a		//Get the accumulator data
	and	r2,r1,0x1FFF
	rol	r3,r1,-13		//Determine the number of samples that have been taken
	and	r3,r3,0x7

	add	r3,r3,-1
	bc	NC,AN_Read3a	//Branch if the first sample has been taken
	
	mov	r3,1			//Set the sample number to 1 and store the first sample
	bra	AN_Read3_S

AN_Read3a:
	add	r3,r3,-1
	bc	NC,AN_Read3b	//Branch if the second sample has been taken
                                                                               
	mov	r3,2			//Set the sample number to 2 and add the second sample
	bra	AN_Read3_S

AN_Read3b:
	add	r3,r3,-1
	bc	NC,AN_Read3c	//Branch if the third sample has been taken

	mov	r3,3			//Set the sample number to 3 and add the third sample
	bra	AN_Read3_S

AN_Read3c:
	add	r3,r3,-1
	bc	NC,AN_Read3d	//Branch if the forth sample has been taken

	mov	r3,4			//Set the sample number to 4 and add the forth sample
	bra	AN_Read3_S

AN_Read3d:
	mov	r3,0			//Clear the accumulator
	st	r3,CSdata1a
	
	add	r1,r2,r0		//Add final sample

	mov	r2,5			//Take the average
	jsr	r6,UnsignedDivide	
	and	r1,r1,0x3FF
	st	r1,CSdata1
	bra	AN_End

AN_Read3_S:	
	rol	r3,r3,13		//Prepare the data for the accumulator
	add	r2,r2,r0
	add	r3,r3,r2
	st	r3,CSdata1a
	bra	AN_Start

AN_End:

	//Restore registers	
	sub	sp, sp, 5
	ld	r6, sp, 4
	ld	r3, sp, 3	
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6