//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Configure_SR.asm
//**       Project:  Mini Sumo Project
//**       Created:  28 Jan 2009 by Josh Sparrow
//**       Revised:  12 Feb 2009 by Josh Sparrow
//**
//**   Description:  Routine to configure ports, timers and so on.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Configure_SR
//	This routine is used to configure the ports, timers
//	and anything else that needs configuring.

Configure_SR:

	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r6, sp, 3
	add	sp, sp, 4

	//Start up the ADC
	mov	r0,0xC4E4		//Start ADC
	outp	r0,ADCcfg0
	
	mov	r0,3500		//Delay
Loop_AN0:
	add	r0,r0,-2
	bc	ZC,Loop_AN0

	mov	r0,0x7F		//Reset ADC
	outp	r0,ADCcfg1
	
	mov	r0,10			//Delay
Loop_AN1:
	add	r0,r0,-2
	bc	ZC,Loop_AN1

	mov	r0,0x3F
	outp	r0,ADCcfg1

	mov	r0,0xC6E4		//Start Sampling
	outp	r0,ADCcfg0

	mov	r0,3500		//Delay
Loop_AN2:
	add	r0,r0,-2
	bc	ZC,Loop_AN2

	//Set up TimerA
	//Set timer to count up to  CRO and Restart at 0
	mov	r0, 0b00111011	// Start Timer (21.333us per tick)
	outp	r0, TMRAcfg

	//TimerA Module 1 - compare mode, input disabled, output enabled, reset/set operation
	//Enable output pin PD5(TMRA1) 
	mov	r0, 0x007F     	
	outp	r0, TMRACCM1cfg
	
	//TimerA Module 2 - compare mode, input disabled, output enabled, reset/set operation
	//Enable output pin PD6(TMRA2) 
	mov	r0, 0x007F     	
	outp	r0, TMRACCM2cfg

	//Set up TimerB
	mov	r0, 0b01111101	// Start Timer (341.333us per tick)
	outp	r0, TMRBcfg
	
	//Now that the modules are configured, lets set the period and initial duty cycle of both modules
	
	//Set the number of steps to count	to _PWM_Period
	//Changing this value changes the period for all the PWM signals (ie they cant each have a different period)
	mov	r0, _PWM_Period	
	outp 	r0, TMRACCR0	
		
			
	//Set the initial Duty cycle of pin PD5(TMRA1)		
	mov	r0, _PWM_Init
	outp	r0, TMRACCR1	
	
	//Set the initial Duty cycle of pin PD6(TMRA2)
	mov	r0, _PWM_Init
	outp	r0, TMRACCR2	

	//Set up PortA
	mov	r0, 0
	outp	r0, GPAcfg

	//Set up PortC
	mov	r0,0x1010
	outp	r0,GPCcfg	

	//Set up PortD
	mov	r0,0x0F00
	outp	r0,GPDcfg
	
	//Set up PortE
	mov	r0,0x0F00
	outp	r0,GPEcfg

	//Set up PortH
	mov	r0,0	
	outp  r0,GPHcfg 
	
	//Set up the serial port

	mov	r1, kXPD_BaudRate_115200 + kXPD_ClockLE_12MHz
	jsr	r6, XPD_Configure

	//Restore registers
	sub	sp, sp, 3
	ld	r6, sp, 2
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6