//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Drive.asm
//**       Project:  Mini Sumo Project
//**       Created:  18 Feb 2009 by Josh Sparrow
//**       Revised:  18 Feb 2009 by Josh Sparrow
//**
//**   Description:  This routine runs the robot along a predefided trajectory while checking
//**			   the status of sensors using the SENcheck subroutine.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Drive
//	This routine is used to move to robot along a
//	predefined trajectory. The trajectory is defined
//	using the input parameters defined below. 
//-------------------------------------------------------
//	Input Params:
//			DRIVEcfg0  -> 8 LSBs = Speed (0-100%)
//					  8 MSBs = Distance
//						     or Radius of Turn (0-256cm)
//			DRIVEcfg1  -> 13 LSBs = Degrees of Turn (0-360degrees)
//					  Next 3 bits = Direction (0=Reverse, 1=Forward, 2=Left, 3=Right, 4=BackLeft, 5=BackRight)
//
//			SENcfg0     -> 0=Don't Check
//					  bit 0 = Set to check if the sensor is set, clear to check if sensor is cleared
//					  bit 1 = Sonar
//					  bit 2 = IR0
//					  bit 3 = IR1
//					  bit 4-7 = Bumpers
//					  bit 8-11 = Line Detectors
//
//			SENcfg1     -> 0=Don't Check
//					  bit 0 = Set to check if the sensor is set, clear to check if sensor is cleared
//					  bit 1 = Sonar
//					  bit 2 = IR0
//					  bit 3 = IR1
//					  bit 4-7 = Bumpers
//					  bit 8-11 = Line Detectors
//
//			SENcfg2     -> 0=Don't Check
//					  bit 0 = Set to check if the sensor is set, clear to check if sensor is cleared
//					  bit 1 = Sonar
//					  bit 2 = IR0
//					  bit 3 = IR1
//					  bit 4-7 = Bumpers
//					  bit 8-11 = Line Detectors
//
//	Output Params:
//			DRIVEdata -> 16 bits = Distance Traveled

Drive:
	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3	
	st	r4, sp, 4
	st	r5, sp, 5	
	st	r6, sp, 6
	add	sp, sp, 7

	mov	r0,0
	st	r0,ECdata0
	st	r0,ECdata1
	st	r0,ECdata0a
	st	r0,ECdata1a
	
	jsr	r6,DelayLong
	//Is the robot going straight or turning	
Start_DR:
	mov	r0,0
	st	r0,PWMdata0
	st	r0,PWMdata1

	//Check if the speed is larger than 100
	ld	r1,DRIVEcfg0
	and	r2,r1,0x00FF
	mov	r0,100
	sub	r2,r2,r0
	bc	NS,Continue_DR
	and	r1,r1,0xFF00
	add	r1,r1,r0
	st	r1,DRIVEcfg0

Continue_DR:
	ld	r0,DRIVEcfg1
	rol	r0,r0,-13
	and	r0,r0,7
	bc	ZS,Reverse_DR
	add	r0,r0,-1
	bc	ZS,Forward_DR
	bra	Turn_DR
	
	//The robot is going in reverse
Reverse_DR:					// r1,r5,r6 are junk registers
	mov	r0,0
	st	r0,Dir0
	st	r0,Dir1

	ld	r3,ECdata0			// r3 = Initial Encoder Data

	ld	r1,DRIVEcfg0		// Set motor speeds
	and	r2,r1,0x00FF
	st	r2,PWMdata0
	st	r2,PWMdata1

	rol	r2,r1,-8
	and	r2,r2,0x00FF		// r2 = Distance

Loop_DR0:
	ld	r4,ECdata0
	sub	r4,r4,r3			// r4 = Distance traveled

	jsr	r6,SENcheck
	ld	r0,SENdata0
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR0
	ld	r0,SENdata1
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR0
	ld	r0,SENdata2
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR0
	
	sub	r1,r4,r2
	bc	NS,Loop_DR0

	add	r1,r4,0x8000		// Distance has been traveled
	bra	Stop_DR

Break_DR0:					// Distance has not been traveled
	mov	r1,r4
	bra	Stop_DR

	
	//The robot is going forward
Forward_DR:					// r1,r5,r6 are junk registers
	mov	r0,1
	st	r0,Dir0
	st	r0,Dir1

	ld	r3,ECdata0			// r3 = Initial Encoder Data

	ld	r1,DRIVEcfg0		// Set motor speeds
	and	r2,r1,0x00FF
	st	r2,PWMdata0
	st	r2,PWMdata1

	rol	r2,r1,-8
	and	r2,r2,0x00FF		// r2 = Distance

Loop_DR1:
	ld	r4,ECdata0
	sub	r4,r4,r3			// r4 = Distance traveled

	jsr	r6,SENcheck
	ld	r0,SENdata0
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR1
	ld	r0,SENdata1
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR1
	ld	r0,SENdata2
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR1
	
	sub	r1,r4,r2
	bc	NS,Loop_DR1

	add	r1,r4,0x8000		// Distance has been traveled
	bra	Stop_DR

Break_DR1:					// Distance has not been traveled
	mov	r1,r4
	bra	Stop_DR


	//The robot is turning
	//Begin by determining the speed and direction
	//of each wheel assuming a left turn. We will
	//make corrections for a right turn later.
Turn_DR:					// r0,r1,r2,r5,r6 are junk registers
	ld	r1,DRIVEcfg0
	and	r3,r1,0x00FF		// r3 = Velocity

	rol	r4,r1,-8
	and	r4,r4,0x00FF		// r4 = Radius of Turn
	add	r4,r4,(_TurnRadius/200)
	add	r0,r4,-6			// Check if radius is less than 5cm
	bc	NC,Skip_DR0

	ld	r0,DRIVEcfg0

	and	r0,r0,0x00FF
	add	r0,r0,0x0500
	st	r0,DRIVEcfg0
	
	mov	r1,r3				// Perform spin about center at the specified speed
	mov	r0,0
	st	r0,Dir0
	mov	r0,1
	st	r0,Dir1
	bra	Set_DR

Skip_DR0:
	add	r0,r4,-(_TurnRadius/100)// Check if radius is 10cm
	bc	ZC,Skip_DR1

	mov	r1,0				// Perform pivot about left tire
	mov	r0,0
	st	r0,Dir0
	mov	r0,1	
	st	r0,Dir1
	bra	Set_DR


	//Begin calculating the velocity of the second wheel
Skip_DR1:
	mov	r1,_TurnRadius
	mov	r2,r4
	jsr	r6,UnsignedDivide

	mov	r0,1				// Assume distance to pivot is larger than _TurnRadius
	st	r0,Dir0
	st	r0,Dir1
	
	mov	r2,100			// Check if the pivot is less than _TurnRadius
	sub	r2,r2,r1
	bc	NC,Skip_DR2
	
	mov	r2,100			// Make corrections for a pivot of less than _TurnRadius
	sub	r2,r1,r2

	mov	r0,0
	st	r0,Dir0
	mov	r0,1
	st	r0,Dir1

	
	//Continue calculating the velocity of the second wheel 
Skip_DR2:
	mov	r1,0
	mov	r0,r3

Multi_DR0:
	add	r1,r1,r2
	add	r0,r0,-1
	bc	ZC,Multi_DR0
	
	mov	r2,100
	jsr	r6,UnsignedDivide

	
	//Now that we have the velocity of the second wheel
	//we should determine if we are turning left, right,
	//backleft or backright
Set_DR:
	ld	r0,DRIVEcfg1		// Determine direction
	rol	r0,r0,-13
	and	r0,r0,7

	add	r0,r0,-2
	bc	ZS,Left_DR

	add	r0,r0,-1
	bc	ZS,Right_DR
		
	add	r0,r0,-1
	bc	ZS,BackLeft_DR

	add	r0,r0,-1
	bc	ZS,BackRight_DR
	bra	Stop_DR
	
BackRight_DR:
	ld	r0,Dir0			// Make corrections for backright turn
	bix	r0,r0,0
	st	r0,Dir0
	
	ld	r0,Dir1
	bix	r0,r0,0
	st	r0,Dir1
	
	//Track encoders until distance has been traveled
Right_DR:
	ld	r0,Dir0			// Make corrections for right turn
	ld	r2,Dir1
	st	r2,Dir0
	st	r0,Dir1

	mov	r0,r1
	mov	r1,r3
	mov	r3,r0

	st	r1,PWMdata0			// Set motor speeds
	st	r3,PWMdata1

	ld	r2,ECdata0			// Record Encoder reading
	
	jsr	r6,Angle_Conv
	mov	r3,r1				// r3 = Distance

Loop_DR2:
	jsr	r6,SENcheck
	ld	r0,SENdata0
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR2
	ld	r0,SENdata1
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR2
	ld	r0,SENdata2
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR2

	ld	r4,ECdata0
	sub	r4,r4,r2			// r4 = Distance traveled
	sub	r1,r4,r3
	bc	NS,Loop_DR2

	mov	r1,r4				// Distance has been traveled
	add	r1,r1,0x8000
	bra	Stop_DR

Break_DR2:					// Distance has not been traveled
	mov	r1,r4
	bra	Stop_DR

BackLeft_DR:
	ld	r0,Dir0			// Make corrections for backleft turn
	bix	r0,r0,0
	st	r0,Dir0
	
	ld	r0,Dir1
	bix	r0,r0,0
	st	r0,Dir1
	
Left_DR:
	st	r1,PWMdata0			// Set motor speeds
	st	r3,PWMdata1
	
	ld	r2,ECdata1			// Record Encoder reading
		
	jsr	r6,Angle_Conv
	mov	r3,r1				// r3 = Distance

Loop_DR3:
	jsr	r6,SENcheck
	ld	r0,SENdata0
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR3
	ld	r0,SENdata1
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR3
	ld	r0,SENdata2
	and	r0,r0,0x0FFE
	bc	ZC,Break_DR3

	ld	r4,ECdata1
	sub	r4,r4,r2			// r4 = Distance traveled
	sub	r1,r4,r3
	bc	NS,Loop_DR3

	mov	r1,r4				// Distance has been traveled
	bis	r1,r1,15
	bra	Stop_DR

Break_DR3:					// Distance has not been traveled
	mov	r1,r4
	bra	Stop_DR

Stop_DR:					// Stop the motors and return the
	st	r1,DRIVEdata		// distance traveled in DRIVEcfg0
	
	ld	r1,ECdata0
	st	r1,ECdata0c
	
	ld	r1,ECdata1
	st	r1,ECdata1c
	
	ld	r1,PWMdata0
	st	r1,PWMdata0a
	ld	r1,PWMdata1
	st	r1,PWMdata1a	
	mov	r0,0
	st	r0,PWMdata0
	st	r0,PWMdata1

//	jsr	r6,Dyn_Brack
	
End_DR:

	//Restore registers
	sub	sp, sp, 7
	ld	r6, sp, 6
	ld	r5, sp, 5		
	ld	r4, sp, 4	
	ld	r3, sp, 3		
	ld	r2, sp, 2		
	ld	r1, sp, 1			
	ld	r0, sp, 0
	
	jsr	r6, r6

//=======================================================
//	Angle_Conv
//	This routine is used by the Drive routine to
//	convert the angle of travel to the distance
//	of travel. 
//-------------------------------------------------------
//	Output Params:	r1 = distance to travel

Angle_Conv:

	// Store registers on stack
	st	r0, sp, 0
	st	r2, sp, 1
	st	r6, sp, 2
	add	sp, sp, 3

	ld	r2,DRIVEcfg1
	and	r2,r2,0x1FFF
	
	mov	r1,0
	mov	r0,63
Multi_AC0:
	add	r1,r1,r2
	add	r0,r0,-1
	bc	ZC,Multi_AC0
	mov	r2,100
	jsr	r6,UnsignedDivide

	ld	r0,DRIVEcfg0		// Determine the distance from angle
	rol	r0,r0,-8
	and	r0,r0,0x00FF
//	add	r0,r0,(_TurnRadius/200)

	mov	r2,0
Multi_AC1:
	add	r2,r2,r1
	add	r0,r0,-1
	bc	ZC,Multi_AC1
	mov	r1,r2
	mov	r2,36
	jsr	r6,UnsignedDivide

	//Restore registers
	sub	sp, sp, 3
	ld	r6, sp, 2
	ld	r2, sp, 1		
	ld	r0, sp, 0
	
	jsr	r6, r6

//=======================================================
//	Dyn_Brack
//	This routine is used by the Drive routine to
//	slow the robot using dynamic bracking. 
//-------------------------------------------------------
//	Output Params:	r1 = distance to travel

Dyn_Brack:
	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3	
	st	r6, sp, 4
	add	sp, sp, 5

	//Invert direction of motors
	ld	r0,Dir0		
	xor	r0,r0,0x0001
	st	r0,Dir0
	
	ld	r0,Dir1
	xor	r0,r0,0x0001
	st	r0,Dir1

	//Set PWM duty cy
Loop_DB0:
	mov	r2,10
	
	ld	r0,ECtime0
	sub	r3,r0,r2
	bc	NS,Right_Check
	st	r0,PWMdata0
	
	ld	r1,ECtime1
	sub	r3,r1,r2
	bc	NS,Left_Check
	st	r1,PWMdata0
	bra	Loop_DB0
	
Right_Check:
	mov	r0,0
	st	r0,PWMdata0

Loop_DB1:
	ld	r1,ECtime1
	sub	r3,r1,r2
	bc	NS,End_DB
	st	r1,PWMdata0
	bra	Loop_DB1

Left_Check:
	mov	r0,0
	st	r0,PWMdata1

Loop_DB2:
	ld	r0,ECtime0
	sub	r3,r0,r2
	bc	NS,End_DB
	st	r0,PWMdata0
	bra	Loop_DB2

End_DB:	

	//Restore registers
	sub	sp, sp, 5
	ld	r6, sp, 4	
	ld	r3, sp, 3		
	ld	r2, sp, 2		
	ld	r1, sp, 1			
	ld	r0, sp, 0
	
	jsr	r6, r6
