//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Encoders.asm
//**       Project:  Quick Start Project
//**       Created:  25 Jun 2002 by Ryan Northcott
//**       Revised:  08 Sep 2006 by Dan Beutel
//**
//**   Description:  Gateway routine to interprate data from the encoders.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descReaded from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Update_Encoders
//	This routine is used to interprate data from the
//	encoders. The distance to an object in cm is
//	placed in ECdata0 and ECdata1.

Update_Encoders:

	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3
	st	r4, sp, 4
	st	r5, sp, 5
	st	r6, sp, 6
	add 	sp, sp, 7

//clear registers
	mov r0,0x0 		//timer
	mov r1,0x0 		//r1 for subroutines, keep clear
	mov r2,0x0 		//2nd timer
	ld  r3,ECstate	//r3 for reading port pins
	ld  r4,ECdata0a	//counterL - pin1 = Left Encoder
	ld  r5,ECdata1a	//counterR - pin2 = Right Encoder

//Read Port
Read:
	inp r0,TMRBcount	//initial read time

	ld  r3,ECstate	//read state
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,6		//11 - both encoder pins logic1
	bc EQ,loop11
	
	ld  r3,ECstate	//read state
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,4		//10 - logic1 pin2, logic0 pin1
	bc EQ,loop10
	
	ld  r3,ECstate	//read state
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,2		//01 - logic0 pin2, logic1 pin1
	bc EQ,loop01
	
	ld  r3,ECstate	//read state
	and r3,r3,6		//read encoders from Port H pins 1 and 2
	sub r3,r3,0		//00 - both encoder pins logic0
	bc EQ,loop00
	

loop11:
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2		
	sub r3,r3,4		//10 - logic1 pin2, logic0 pin1
	bc EQ,counterL
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,2		//01 - logic0 pin2, logic1 pin1
	bc EQ,counterR
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,0		//00 - boh encoder pins logic0
	bc EQ,counterall
	
	bra loop11
	
loop10:
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2		
	sub r3,r3,6		//11 - both encoder pins logic1
	bc EQ,counterL
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,2		//01 - logic0 pin2, logic1 pin1
	bc EQ,counterall
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,0		//00 - boh encoder pins logic0
	bc EQ,counterR
	
	bra loop10
	
loop01:
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,6		//11 - both encoder pins logic1
	bc EQ,counterR
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,4		//10 - logic1 pin2, logic0 pin1
	bc EQ,counterall
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,0		//00 - boh encoder pins logic0
	bc EQ,counterL
	
	bra loop01
	
loop00:
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,6		//11 - both encoder pins logic1
	bc EQ,counterall
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,4		//10 - logic1 pin2, logic0 pin1
	bc EQ,counterR
	
	inp r3,GPHin	//read port H
	and r3,r3,6		//read encoders from Port H pins 1 and 2	
	sub r3,r3,2		//01 - logic0 pin2, logic1 pin1
	bc EQ,counterL
	
	bra loop00
	
//counter for encoder at pin1 = left
counterL:
	inp r3,GPHin	//read port H
	st  r3,ECstate    //record port H
	inp r2,TMRBcount	//2nd time
	sub r2,r2,r0
	sub r2,r2,6	//2.75msec
	//sub r2,r2,214	//5msec
	bc NS, End_EC

	add r4,r4,1
	st r4,ECdata0a

	
	bra End_EC
	
//counter for encoder at pin2 = right
counterR:
	inp r3,GPHin	//read port H
	st  r3,ECstate    //record port H
	inp r2,TMRBcount	//2nd time
	sub r2,r2,r0
	sub r2,r2,6	//2.75msec
	//sub r2,r2,214	//5msec
	bc NS, End_EC
	
	add r5,r5,1
	st r5,ECdata1a

	
	bra End_EC

//counter for both encoders (right and left)
counterall:
	inp r3,GPHin	//read port H
	st  r3,ECstate    //record port H
	inp r2,TMRBcount	//2nd time
	sub r2,r2,r0
	sub r2,r2,4	//2.75msec
	//sub r2,r2,214	//5msec
	bc NS, End_EC

	add r4,r4,1
	st r4,ECdata0a
	
	add r5,r5,1
	st r5,ECdata1a

	bra End_EC

End_EC:	
	sub	sp, sp, 7
	ld	r6, sp, 6
	ld	r5, sp, 5
	ld	r4, sp, 4
	ld	r3, sp, 3
	ld	r2, sp, 2
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6
	
/////////////////////////////////////////////////////////////////////////////////////////////////	
//Subroutines	
/////////////////////////////////////////////////////////////////////////////////////////////////
//Enc_Conv subroutine is used to convert encoder transitions to distance travelled by encoder wheel
//
//Calculation details: 	wheel diameter = 42mm = 4.2cm,
//				24 transitions/rotation
//				rotation = circumference = pi*d = pi*4.2cm = 13.19469 cm
//				distance travelled/transition = 13.1949cm/24 = 0.549779cm/transition
//
//
/////////////////////////////////////////////////////////////////////////////////////////////////
Enc_Conv:
		st	r1, sp, 0
		st	r2, sp, 1
		st	r3, sp, 2
		st	r4, sp, 3
		st	r5, sp, 4
		st	r6, sp, 5
		add 	sp, sp, 6
		
		ld r4, ECdata0a
		ld r5, ECdata1a
	
		//0.549779cm/transition = appx. 10/18
		mov r2, 10
		mov r3, 0	
	LeftLoop:	
		add r3,r3,r4 	//0.549779 = cm/transition 
		sub r2, r2, 1
		bc ZC, LeftLoop
		
		mov r1, r3
		mov r2, 18
		
		jsr r6,UnsignedDivide
		st r1, ECdata0		
		
		mov r2, 10
		mov r3, 0
	RightLoop:	
		add r3,r3,r5 	
		sub r2, r2, 1
		bc ZC, RightLoop
		
		mov r1, r3
		mov r2, 18
		
		jsr r6,UnsignedDivide
		st r1, ECdata1

	//restore
		sub	sp, sp, 6
		ld	r6, sp, 5
		ld	r5, sp, 4
		ld	r4, sp, 3
		ld	r3, sp, 2
		ld	r2, sp, 1
		ld	r1, sp, 0
		jsr	r6, r6	
//////////////////////////////////////////////////////////////////////////////////////////////////

		

	bra	@	// Prevent this thread from running away if it is accidentally enabled in the main file