//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Hunt_P1.asm
//**       Project:  Mini Sumo Project
//**       Created:  10 Jan 2009 by Josh Sparrow
//**       Revised:  12 Feb 2009 by Josh Sparrow
//**
//**   Description:  Routines to control 2 motors using PWM.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Hunt_P1
//	This routine is used operate the robot in a
//	hunting maner. When the robot locks on it
//	will go to attack. If the robot encounters
//	a bumper or line detector it will exit.

Hunt_P1:
		
	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2		//Direction
	st	r6, sp, 3
	add	sp, sp, 4
		
Start_HU:
	//Check semsors
	mov	r0,0x0FF1
	st	r0,SENcfg0
	
	mov	r0,0x000F
	st	r0,SENcfg1
	
	jsr	r6,SENcheck
	
	//Check Defensive sensors
	ld	r0,SENdata0
	and	r0,r0,0x0FF0
	bc	ZC,End_HU

	//Determine state
	ld	r0,SENdata1
	add	r1,r0,-3
	bc	ZS,End_HU

	add	r1,r0,-15
	bc	ZS,End_HU

	add	r1,r0,-9
	bc	ZS,S4_HU

	add	r1,r0,-11
	bc	ZS,S3_HU

	add	r1,r0,-7
	bc	ZS,S2_HU

	add	r1,r0,-5
	bc	ZS,S1_HU

S0_HU:
	mov	r0,0x000F		//Monitor the sonar and IR sensors
	st	r0,SENcfg1

	mov	r0,0			//Turn off last register
	st	r0,SENcfg2

	mov	r2,3			//Turn right
	jsr	r6,Set_Drive_HU
	bra	Start_HU

S1_HU:
	mov	r0,0x0004		//Check if IR0 is cleared
	st	r0,SENcfg1
	
	mov	r0,0x000B		//Check if SN or IR1 are set
	st	r0,SENcfg2

	mov	r2,2			//Turn left
	jsr	r6,Set_Drive_HU
	bra	Start_HU

S2_HU:
	mov	r0,0x0006		//Check if SN or IR0 are cleared
	st	r0,SENcfg1
	
	mov	r0,0x0009		//Check if IR1 is set
	st	r0,SENcfg2

	mov	r2,2			//Turn left
	jsr	r6,Set_Drive_HU
	bra	Start_HU

S3_HU:
	mov	r0,0x000A		//Check if SN or IR1 are cleared
	st	r0,SENcfg1
	
	mov	r0,0x0005		//Check if IR0 is set
	st	r0,SENcfg2

	mov	r2,3			//Turn right
	jsr	r6,Set_Drive_HU
	bra	Start_HU

S4_HU:
	mov	r0,0x0008		//Check if IR1 is cleared
	st	r0,SENcfg1
	
	mov	r0,0x0007		//Check if SN or IR0 are set
	st	r0,SENcfg2

	mov	r2,3			//Turn right
	jsr	r6,Set_Drive_HU
	bra	Start_HU
			
End_HU:				//Should be centered on the opponent unless defensive sensor has been activated
	//Restore registers
	sub	sp, sp, 4
	ld	r6, sp, 3
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6
	
//=======================================================
//	Set_Drive_HU
//	This routine is used set the Drive registers
//	according to r2.

Set_Drive_HU:

	// Store registers on stack
	st	r0, sp, 0		//Junk
	st	r1, sp, 1		//Junk
	st	r6, sp, 2
	add	sp, sp, 3

	mov	r0,100		//Set speed to 100%
	mov	r1,0			//Set radius to pivot
	rol	r1,r1,8
	add	r0,r0,r1
	st	r0,DRIVEcfg0

	mov	r0,720		//Set turn angle to 720
	rol	r1,r2,13		//Adjust direction register
	add	r0,r0,r1
	st	r0,DRIVEcfg1
	jsr	r6,Drive

	//Restore registers
	sub	sp, sp, 3
	ld	r6, sp, 2
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6