//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Motors.asm
//**       Project:  Mini Sumo Project
//**       Created:  10 Jan 2009 by Josh Sparrow
//**       Revised:  12 Feb 2009 by Josh Sparrow
//**
//**   Description:  Routines to control 2 motors using PWM.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Update_Motors
//	This routine is used to update the speed and 
//	direction of the two motors.
//	The latest data in PWMdata0 and PWMdata1 are used
//	to update the speed. The latest data in Dir0 and
//	Dir1 are used to set the direction.

Update_Motors:
		
	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r6, sp, 3
	add	sp, sp, 4
	

	//Set the direction of motor0 based on contents of "Dir0"
Check_Dir0:	
	ld	r1,Dir0
	bc	ZS,Motor0_Rev

	inp	r2,GPEin		//Set output to forward
	and	r2,r2,0xFFFC
	add	r2,r2,0b01
	outp	r2,GPEout
	
	bra Check_Dir1
		
Motor0_Rev:
	inp	r2,GPEin		//Set output to reverse
	and	r2,r2,0xFFFC
	add	r2,r2,0b10
	outp	r2,GPEout
			
		
	//Set the direction of motor1 based on contents of "Dir1"
Check_Dir1:	
	ld	r1,Dir1
	xor	r1,r1,0x0001
	bc	ZS,Motor1_Rev

	inp	r2,GPEin		//Set output to forward
	and	r2,r2,0xFFF3
	add	r2,r2,0b0100
	outp	r2,GPEout

	bra Check_Duty0
		
Motor1_Rev:
	inp	r2,GPEin		//Set output to reverse
	and	r2,r2,0xFFF3
	add	r2,r2,0b1000
	outp	r2,GPEout
	
	//Deal with a duty cycle of 0 or more than 100
Check_Duty0:
	ld	r1,PWMdata0
	add	r1,r1,r1
	bc	ZC,Duty_Full0
		
Duty_Zero0:
	inp	r2,GPEin		//Set output to 0
	and	r2,r2,0xFFFC
	outp	r2,GPEout
	bra	Check_Duty1
		
Duty_Full0:
	mov	r0,_PWM_Period
	sub	r0,r0,1
	sub	r2,r0,r1
	bc	NC,Check_Duty1

	st	r0,PWMdata0

Check_Duty1:
	ld	r1,PWMdata1
	bc	ZC,Duty_Full1
		
Duty_Zero1:
	inp	r2,GPEin
	and	r2,r2,0xFFF3
	outp	r2,GPEout
	bra	SetPWM
	
Duty_Full1:
	mov	r0,_PWM_Period
	sub	r0,r0,1
	sub	r2,r0,r1
	bc	NC,SetPWM

	st	r0,PWMdata1
					
	//Now that the direction of each motor is set, lets update the PWM duty cycle
SetPWM:	
	ld	r0,PWMdata0
	outp	r0,TMRACCR1		//set the new duty cycle of TMRA1
		
	ld	r0,PWMdata1
	outp	r0,TMRACCR2		//set the new duty cycle of TMRA2
	
		
	//Restore registers
	sub	sp, sp, 4
	ld	r6, sp, 3
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6