//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  SENcheck.asm
//**       Project:  Mini Sumo Project
//**       Created:  18 Feb 2009 by Josh Sparrow
//**       Revised:  7 Mar 2009 by Josh Sparrow
//**
//**   Description:  This routine to is used to check the status of the sensors defined
//**			   in SENcfg. The result is put in SENdata.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	SENcheck
//	This routine is used to check the sensors which
//	has been specified in SENcfg and put the result
//	in SENdata.
// 
//-------------------------------------------------------
//	Input Params:
//			SENcfg0     -> 0=Don't Check - Returns 0
//					  bit 0 = Set to check if the sensor is set, clear to check if sensor is cleared
//					  bit 1 = Sonar
//					  bit 2 = IR0
//					  bit 3 = IR1
//					  bit 4-7 = Bumpers
//					  bit 8-11 = Line Detectors
//
//			SENcfg1     -> 0=Don't Check - Returns 0
//					  bit 0 = Set to check if the sensor is set, clear to check if sensor is cleared
//					  bit 1 = Sonar
//					  bit 2 = IR0
//					  bit 3 = IR1
//					  bit 4-7 = Bumpers
//					  bit 8-11 = Line Detectors
//
//	Output Params:
//			SENdata0    -> Any condition above that is true for SENcfg0.
//
//			SENdata1    -> Any condition above that is true for SENcfg1.

SENcheck:
	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3		//SENdata register
	st	r4, sp, 4		//Loop count
	st	r6, sp, 5
	add	sp, sp, 6

	mov	r4,3
Start_SC:
	mov	r3,0
	add	r4,r4,-1
	bc	ZS,Check2_SC
	
	add	r1,r4,-1
	bc	ZS,Check1_SC
	
	ld	r1,SENcfg0
	bc	ZS,Store_SC0
	bra	Check_SC

Check1_SC:
	ld	r1,SENcfg1
	bc	ZS,Store_SC0
	bra	Check_SC

Check2_SC:
	ld	r1,SENcfg2
	bc	ZS,Store_SC0
	
Check_SC:
	and	r2,r1,1
	mov	r3,r2
	xor	r2,r2,1
	
SNcheck_SC:
	and	r0,r1,0x0002
	bc	ZS,IR0check_SC
	
	ld	r0,SNdata
	add	r0,r0,1
	bc	ZS,SNcl_SC

	mov	r0,1
	bra	SNend_SC
	
SNcl_SC:
	mov	r0,0

SNend_SC:
	xor	r0,r0,r2
	rol	r0,r0,1
	add	r3,r3,r0

IR0check_SC:
	and	r0,r1,0x0004
	bc	ZS,IR1check_SC
	
	ld	r0,IRdata0
	add	r0,r0,-1023
	bc	ZS,IR0cl_SC

	mov	r0,1
	bra	IR0end_SC
	
IR0cl_SC:
	mov	r0,0

IR0end_SC:
	xor	r0,r0,r2
	rol	r0,r0,2
	add	r3,r3,r0

IR1check_SC:
	and	r0,r1,0x0008
	bc	ZS,SW0check_SC
	
	ld	r0,IRdata1
	add	r0,r0,-1023
	bc	ZS,IR1cl_SC

	mov	r0,1
	bra	IR1end_SC
	
IR1cl_SC:
	mov	r0,0

IR1end_SC:
	xor	r0,r0,r2
	rol	r0,r0,3
	add	r3,r3,r0

SW0check_SC:
	and	r0,r1,0x0010
	bc	ZS,SW1check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0001

	xor	r0,r0,r2
	rol	r0,r0,4
	add	r3,r3,r0

SW1check_SC:
	and	r0,r1,0x0020
	bc	ZS,SW2check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0002
	rol	r0,r0,-1
	
	xor	r0,r0,r2
	rol	r0,r0,5
	add	r3,r3,r0

SW2check_SC:
	and	r0,r1,0x0040
	bc	ZS,SW3check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0004
	rol	r0,r0,-2

	xor	r0,r0,r2
	rol	r0,r0,6
	add	r3,r3,r0

SW3check_SC:
	and	r0,r1,0x0080
	bc	ZS,LD0check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0008
	rol	r0,r0,-3

	xor	r0,r0,r2
	rol	r0,r0,7
	add	r3,r3,r0

LD0check_SC:
	and	r0,r1,0x0100
	bc	ZS,LD1check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0010
	rol	r0,r0,-4

	xor	r0,r0,r2
	rol	r0,r0,8
	add	r3,r3,r0

LD1check_SC:
	and	r0,r1,0x0200
	bc	ZS,LD2check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0020
	rol	r0,r0,-5

	xor	r0,r0,r2
	rol	r0,r0,9
	add	r3,r3,r0

LD2check_SC:
	and	r0,r1,0x0400
	bc	ZS,LD3check_SC
	
	ld	r0,DEFdata
	and	r0,r0,0x0040
	rol	r0,r0,-6

	xor	r0,r0,r2
	rol	r0,r0,10
	add	r3,r3,r0

LD3check_SC:
	and	r0,r1,0x0800
	bc	ZS,Store_SC0
	
	ld	r0,DEFdata
	and	r0,r0,0x0080
	rol	r0,r0,-7

	xor	r0,r0,r2
	rol	r0,r0,11
	add	r3,r3,r0

Store_SC0:
	and	r3,r3,0x0FFF

	mov	r4,r4
	bc	ZS,Store_SC2
	
	add	r1,r4,-1
	bc	ZS,Store_SC1
	
	st	r3,SENdata0
	bra	Start_SC

Store_SC1:
	st	r3,SENdata1
	bra	Start_SC

Store_SC2:
	st	r3,SENdata2

End_SC:	
	//Restore registers
	sub	sp, sp, 6
	ld	r6, sp, 5
	ld	r4, sp, 4
	ld	r3, sp, 3
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6