//*************************************************************************************************
//************************ (C) 2001 - 2006, Eleven Engineering Incorporated ***********************
//*************************************************************************************************
//**
//**		Tabs:	This file looks best with tab stops set every 6 spaces.
//**
//*************************************************************************************************
//*************************************************************************************************
//**
//**          File:  Sonar.asm
//**       Project:  Mini Sumo Project
//**       Created:  12 Jan 2009 by Josh Sparrow
//**       Revised:  12 Feb 2009 by Josh Sparrow
//**
//**   Description:  This file contains the application code that will be operate the sonar.
//**
//** Compatibility:  Requires XInA v4.4 or higher.
//**
//**    Disclaimer:  You may incorporate this sample source code into your program(s) without
//**                 restriction.  This sample source code has been provided "AS IS" and the
//**                 responsibility for its operation is yours.  You are not permitted to
//**                 redistribute this sample source code as "Eleven sample source code" after
//**                 having made changes.  If you are going to re-distribute the source, we require
//**                 that you make it clear in the source that the code was descended from Eleven
//**                 sample source code, but that you have made changes.
//**
//*************************************************************************************************
//*************************************************************************************************

//=======================================================
//	Update_Sonar
//	This routine is used to operate the sonar. The
//	distance to an object in mm is placed in SNdata.

Update_Sonar:

	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2
	st	r3, sp, 3	
	st	r6, sp, 4
	add	sp, sp, 5

	mov	r0,0x0010		//Charge output to 1
	outp	r0,GPCout
	
Start_SN:

	//Send an inverted trigger pulse

	mov	r0,0			//Set output to 0
	outp	r0,GPCout

	mov	r0,_SN_trigger	//Trigger pulse length
Loop_SN0:	
	add	r0,r0,-2
	bc	NC,Loop_SN0	

	mov	r0,0x0010		//Set output to 1
	outp	r0,GPCout

	mov	r0,1040		//Delay
Loop_SN1:	
	add	r0,r0,-2
	bc	NC,Loop_SN1	


	//Wait for rising edge signal from sonar
	mov	r0,_SN_time
Loop_SN2:
	add	r0,r0,-5	
	bc	NS,Start_SN
	
	inp	r1,GPCin
	and	r1,r1,0x0020
	bc	ZS,Loop_SN2


	//Count till falling edge signal from sonar
	mov	r0,_SN_time
Loop_SN3:
	add	r0,r0,-5
	bc	NS,Start_SN
	
	inp	r1,GPCin
	and	r1,r1,0x0020
	bc	ZC,Loop_SN3

	//Convert the result into mm
	mov	r1,_SN_time	
	sub	r1,r1,r0
	mov	r2,_SN_devider
	jsr	r6,UnsignedDivide

	
	//Check if the result is lower than _SN_min
	sub	r2,r1,_SN_min
	bc	NC,Skip_SN
	
	mov	r1,0
	bra	Record_SN
	
	
	//Check if the result is larger than _SN_max
Skip_SN:
	sub	r2,r1,_SN_max
	bc	NS,Record_SN
	
	mov	r1,0xFFFF

	
	//Record the data to ram	
Record_SN:
	st	r1,SNdata1

	//Wait till the end of _SN_time before returning
Loop_SN4:
	add	r0,r0,-2
	bc	NC,Loop_SN4

	//Restore registers
	sub	sp, sp, 5
	ld	r6, sp, 4
	ld	r3, sp, 3		
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6

//=======================================================
//	Conv_Sonar
//	This routine is used convert the sonar data to mm.

Conv_Sonar:

	// Store registers on stack
	st	r0, sp, 0
	st	r1, sp, 1
	st	r2, sp, 2	
	st	r6, sp, 3
	add	sp, sp, 4

	ld	r0,SNdata1
	add	r1,r0,1
	bc	ZS,Full_CVS
	mov	r1,0
	mov	r2,12
Multi_CVS:
	add	r1,r1,r0
	add	r2,r2,-1
	bc	ZC,Multi_CVS
	
	mov	r2,10
	jsr	r6,UnsignedDivide
	st	r1,SNdata
	bra	End_CVS
Full_CVS:	
	st	r0,SNdata
End_CVS:
	//Restore registers	
	sub	sp, sp, 4
	ld	r6, sp, 3
	ld	r2, sp, 2		
	ld	r1, sp, 1
	ld	r0, sp, 0
	
	jsr	r6, r6